/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$

 Add 29/03/01 Pierre Reboours IFEOMesh Functions to localize the nodes
 Add 12/06/01 Pierre Rebours IFEOMesh IsSideFixed
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_fem_interfaces_h_
#define _hyp_fem_interfaces_h_

#define hyp_NAMESPACE_BEGIN_HYPFEM hyp_NAMESPACE_BEGIN_HYP \
										hyp_NAMESPACE_BEGIN(hypfem)
#define hyp_NAMESPACE_END_HYPFEM hyp_NAMESPACE_END_HYP \
										hyp_NAMESPACE_END(hypfem)


hyp_NAMESPACE_BEGIN_HYPFEM


//-----------------------------------------------------------------------
/*! \defgroup Ephy_Interface_Common Common Interfaces
\ingroup Ephy_Interface
@{
*/

/*!
Defines the basic functionnalities of a \ref Ephy_G_OrientedContainerInterface , that is, a set of references
to component interfaces.
In the following documentation, \e container reference to the component which implement this interface.

\HypInvariant
\arg  The container doesn't handle copies of components but only references to the components.
So the implementation must increment (for the insertion) or decrement (for the remove) the counting reference of the interface.
\arg The implementation could check if the inserted component supports other interfaces
 else than the compulsory IUnknown interface.

\HypKey
\arg \ref Ephy_G_OrientedContainerInterface

\HypIID
IID_hypIContainer {CC11C7C1-9F7E-11d4-A08F-0080AD97951B}
*/
struct IContainer : 
public hyp_ker::IUnknown
{
	/*!
	Adds a component in the container.
	\param pObject Component inserted
	*/
	virtual void Add(hyp_ker::IUnknown* pObject) =0;
	/*!
	Removes all the components attached to the container.
	*/
	virtual void RemoveAll() =0;
	/*!
	Returns the number of components in the container.
	*/
	virtual t_size GetSize() const =0;
	/*!
	Removes a component from the container.
	\param pObject Component removed	
	*/
	virtual void Remove(hyp_ker::IUnknown* pObject) =0;
};

hyp_DEFINE_GUID(IID_hypIContainer, 
0xcc11c7c1, 0x9f7e, 0x11d4, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IContainer*/
typedef hyp_ker::IPtr<IContainer,&IID_hypIContainer> t_spContainer;

/*!
Because the IContainer doesn't provide service to get back a reference to the component previoulsy inserted,
several interfaces have been provided.
IContainerKey, an \ref Ephy_G_OrientedContainerInterface, enables to associate to each inserted component
a unique number called \ref Ephy_G_ContainerLabel.

\HypInvariant
- The implementation is warranted the unicity of such label in a container.
- Consequently, if the client inserts a component associated with a label already existing, the
implementation must thrown a warning at least.
- The implementation doesn't increment the reference count when \ref GetContent is called.

\HypKey
- \ref Ephy_G_OrientedContainerInterface
- \ref Ephy_G_ContainerLabel

\HypIID
IID_hypIContainerKey {CC11C7C2-9F7E-11d4-A08F-0080AD97951B}
*/
struct IContainerKey :
public hyp_ker::IUnknown
{
	/*!
	Adds a component to the container.
	\param pObject Inserted component
	\param Label \ref ContainerLabel of the inserted component
	*/
	virtual	void Add(hyp_ker::IUnknown* pObject,const t_label& Label) =0;
	/*!
	Removes a component from the container.
	\param Label \ref ContainerLabel of the removed component
	*/
	virtual	void Remove(const t_label& Label) =0;
	/*!
	Returns  a reference to a component.
	\param Label \ref ContainerLabel of the accessed component
	*/
	virtual hyp_ker::IUnknown* GetContent(const t_label& Label) =0;
	/*!
	Returns the \ref ContainerLabel of a component.
	\param pObject Component associated to the \ref ContainerLabel
	*/
	virtual t_label GetKey(hyp_ker::IUnknown* pObject) =0;
	/*!
	Modifies the label of an inserted component.
	\param pObject Component
	\param Label New \ref ContainerLabel
	*/
	virtual void SetKey(hyp_ker::IUnknown* pObject,const t_label& Label) =0;
};

hyp_DEFINE_GUID(IID_hypIContainerKey, 
0xcc11c7c2, 0x9f7e, 0x11d4, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IContainerKey*/
typedef hyp_ker::IPtr<IContainerKey,&IID_hypIContainerKey> t_spContainerKey;

/*!
IContainerIterator provides the notion of \e iteration and \e sequence.
The interface, an \ref Ephy_G_OrientedContainerInterface, doesn't provide services to sort the sequence.

\HypInvariant
- The implementation generally defines an intern iterator as, for instance, a File Object.

\HypKey
- \ref Ephy_G_OrientedContainerInterface

\HypIID
IID_hypIContainerIterator {CC11C7C3-9F7E-11d4-A08F-0080AD97951B}
*/
struct IContainerIterator :
public hyp_ker::IUnknown
{
	/*!
	Returns the beginning of the sequence.
	*/
	virtual hyp_ker::IUnknown* SetBeginSelection() =0;
	/*!
	Returns the current component of the sequence.
	*/
	virtual	hyp_ker::IUnknown* GetSelection() =0;
	/*!
	Increments the pointer of the sequence. Afterwards, returns \ref GetSelection.
	*/
	virtual hyp_ker::IUnknown* IncrSelection() =0;
};

hyp_DEFINE_GUID(IID_hypIContainerIterator, 
0xcc11c7c3, 0x9f7e, 0x11d4, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IContainerIterator*/
typedef hyp_ker::IPtr<IContainerIterator,&IID_hypIContainerIterator> t_spContainerIterator;

/*!
The creation of a component is slightly longer than a direct call to the new operator for instance.
To avoid this problem, one of the solutions is to keep a reference to the factory which 
has created the component.
IFactoriesLibrary is an interface which allows to manage a huge bulk of factories.

\HypInvariant

\HypKey

\HypIID
IID_hypIFactoriesLibrary {143B7161-196C-11d5-A090-0080AD97951B}
*/
struct IFactoriesLibrary :
public hyp_ker::IUnknown
{
	/*!
	Creates a new component. See the function \ref hypker::CreateInstance for more information.
	\param CLSID CLSID of the new component	
	*/
	virtual void CreateInstance(const hyp_ker::hypCLSID& CLSID,
		hyp_ker::IUnknown* pIUnknownOuter,
		t_ulong,
		const hyp_ker::hypIID&,
		void**) =0;

	/*!
	\overload
	\param FriendlyName Friendly name of the component
	*/
	virtual void CreateInstance(const char* FriendlyName,
		hyp_ker::IUnknown*,
		t_ulong,
		const hyp_ker::hypIID&,
		void**) =0;
};

hyp_DEFINE_GUID(IID_hypIFactoriesLibrary, 
0x143b7161, 0x196c, 0x11d5, 0xa0, 0x90, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IFactoriesLibrary*/
typedef hyp_ker::IPtr<IFactoriesLibrary,&IID_hypIFactoriesLibrary> t_spFactoriesLibrary;

/*!@}*/
//-----------------------------------------------------------------------


//-----------------------------------------------------------------------
/*! \defgroup Ephy_Interface_Math Math Interfaces
\ingroup Ephy_Interface
@{
*/

/*!
The FEM is based on the algebraic equations and also on matrix computations.
The \ref Ephy_G_Ephydryne provide a single interface to deal with the matrices.
  
In the following documentation, the matrix A is referencied as the component.

\HypInvariant
\arg The first row and the first column correspond to the number 0. So the interface follows
the \ref Ephy_G_CArray.
\arg It is up to the implementation to optimize the matrix component. A \ref Ephy_G_MatrixComponent,
which has only one row (i.e. a vector) can internally use a one-dimension array for instance.

\HypKey
\arg \ref Ephy_G_MatrixComponent
\arg \ref Ephy_G_CArray

\HypIID
IID_hypIMathMatrix {BF8B3F63-1F01-11d5-A095-0080AD97951B}
*/
struct IMatrix :
public hyp_ker::IUnknown  
{
	/*!
	B=A.
	\param B Matrix B
	*/
	virtual void SetMatrix(IMatrix* B) =0;
	/*!
	Extracts some elements of the matrix A to the Matrix B. The elements are delimited by a
	square defined itself by four coordinates. The boundaries of the square are extracted as well.
	\param MinX Lower column extracted
	\param MaxX Upper column extracted
	\param MinY Lower row extracted
	\param MaxY Upper row extracted
	\param B Matrix B
	\invariant MinX<=MaxX and MinY<=MaxY
	*/
	virtual void Extract(const t_size& MinX,const t_size& MaxX,const t_size& MinY, const t_size& MaxY, IMatrix* B) =0;
	/*!
	Increments one of the components of the Matrix A. A[X][Y]+=Scalar.
	\param Column of the component
	\param Row of the component
	\param Scalar Real from which the component of the matrix is increased.
	*/
	virtual void IncrValue(const t_size& X, const t_size& Y, const t_real& Scalar) =0;
	/*!
	Sets the dimension of the matrix A.
	\param Column Number of columns
	\param Row Number of rows
	\param Scalar Real which is used to initialize all the components of the matrix	after its size has been changed.
	*/
	virtual void SetDim(const t_size& Column, const t_size& Row,const t_real& Scalar=0) =0;
	/*!
	Sets one of the components of the Matrix A. A[X][Y]=Scalar.
	\param X Column of the component
	\paran Y Row of the component
	\param Scalar Real from which the component of the matrix is initialized
	*/
	virtual void SetValue(const t_size& X, const t_size& Y, const t_real& Scalar) =0;
	/*!
	Retrieves one of the components of the Matrix A. Scalar=A[X][Y].
	\param X Column of the component
	\param Y Row of the component
	\return Value of the component of the matrix
	*/
	virtual t_real GetValue(const t_size& X, const t_size& Y)const =0;
	/*!
	Initializes all the components of the matrix to a same value. A=Scalar.
	\param Scalar Real which uses to initializes the components of the matrix
	*/
	virtual void SetMatrix(const t_real& Scalar) =0;
	/*!
	Gets the number of rows of the Matrix A.
	\return Number of rows of the matrix
	*/
	virtual t_size GetNbRow() const =0;
	/*!
	Gets the number of columns of the Matrix A.
	\return Number of columns of the matrix
	*/
	virtual t_size GetNbColumn() const =0;
	/*!
	Multiplies all the component of the Matrix A by a scalar. A*=Scalar.
	\param Scalar Real which multiplies all the components of the matrix
	*/
	virtual void Multiply(const t_real& Scalar) =0;
	/*!
	Multiplies two Matrices and stockes the result in another Matrix. A*B=C.
	\param B Matrix B
	\param Matrix C
	*/
	virtual void Multiply(const IMatrix* B, IMatrix* C) =0;
	/*!
	Substracts the Matrix A by the Matrix B. A-=B.
	\param Matrix B
	*/
	virtual void Substract(const IMatrix* B) =0;
	/*!
	Inverts the Matrix A. A=A-1.
	*/
	virtual void Invert() =0;
	/*!
	Adds the Matrix B to the Matrix A. A+=B.
	\param B Matrix B
	*/
	virtual void Add(const IMatrix* B) =0;
	/*!
	Transposes the Matrix A and stocke the result in the Matrix B. B=Transpose(A).
	\param B Matrix B
	*/
	virtual void Transpose(IMatrix* B)=0;
	/*!
	Ckecks whether the Matrix A is symetric. A.Transpose(A)==Identity.
	*/
	virtual bool IsSymmetrical() =0;
	/*!
	Erases one of the rows of the Matrix A.
	\param Y Number of the erased row
	*/
	virtual void EraseRow(const t_size& Y) =0;
	/*!
	Erases one of the columns of the Matrix A.
	\param X Number of the erased column
	*/
	virtual void EraseColumn(const t_size& X) =0;
	/*!
	Erases some of the columns and the rows of the Matrix A.
	\param ColumnMask Mask indicates which column must be erased. If ColumnMask={1,0,0,1}, the Column number 0 and 3
	must be erased.
	\param RowMask Mask indicates which row must be erased
	*/
	virtual void EraseRowsAndColumns(const t_mask_array& RowMask ,const t_mask_array& ColumnMask) =0;
};

hyp_DEFINE_GUID(IID_hypIMathMatrix, 
0xbf8b3f63, 0x1f01, 0x11d5, 0xa0, 0x95, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IMatrix*/
typedef hyp_ker::IPtr<IMatrix,&IID_hypIMathMatrix> t_spMathMatrix;

/*!@}*/
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
/*! \defgroup Ephy_Interface_Data Data Interfaces
\ingroup Ephy_Interface
@{
*/

/*!
\HypGeomData
Defines the notion of geometrical point, referred to Vertex as well.

The interface permits to define multi-dimentional points:
\arg One dimension, the system is a line;
\arg Two dimensions, the system is a plan;
\arg Three dimensions, the system is a volume.

Since \ref Ephy_G_Ephydryne are supposed to use for 3D physics engines, the components implement the
3D vertex.
So the vertex has three cordinates, each coordinate is identified by a label called \ref Ephy_G_CoordinateLabel:
\arg X coordinate corresponds to the \ref Ephy_G_CoordinateLabel 0;
\arg Y coordinate corresponds to the \ref Ephy_G_CoordinateLabel 1;
\arg Z coordinate corresponds to the \ref Ephy_G_CoordinateLabel 2.

\HypInvariant

\HypKey
\arg \ref Ephy_G_3DSystem
\arg \ref Ephy_G_CoordinateLabel

\HypIID
IID_hypIGeometricVertex {196913E5-18BD-11d5-A08F-0080AD97951B}
*/
struct IGeometricVertex :
public hyp_ker::IUnknown
{
	/*!
	Gets one of the coordinates of the vertex.
	\param Label Coordinate label 
	*/
	virtual t_real GetCoord(const t_size& Label) const =0;
	/*!
	Sets one of the coordiantes of the vertex.
	\param Scalar New coordinate of the vertex
	\param Label Coordinate label
	*/
	virtual void SetCoord(const t_real& Scalar,const t_size& Label) =0;
	/*!
	Checks if the vertex has a visible property. The Visible Property could be interpreted
	differently by several implementation.
	*/
	virtual bool IsVisible() const =0;
	/*!
	Sets the visble property of the vertex.
	\param bVisible Value of the visible property.
	*/
	virtual void Visible(bool bVisible) =0;
};

hyp_DEFINE_GUID(IID_hypIGeometricVertex, 
0x196913e5, 0x18bd, 0x11d5, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IGeometricVertex*/
typedef hyp_ker::IPtr<IGeometricVertex,&IID_hypIGeometricVertex> t_spGeometricVertex;

/*!
\HypGeomData
Base class which provides some features commun to several \ref Ephy_G_GeometricData.
Actually a object, whose its dimension is upper than one, could be described by a
cluster of vertices. For instance the cube could be defined by 6 vertices plus some
pieces of information to known, for instance, how to connect these vertices together.

\par
In this package we suppose that is enough to give a unique number to each vertex to
rebuild the basic shapes like a cube from scratch.
That's why each vertices inserted has a label, called \ref Ephy_G_VertexLabel.

\HypInvariant

\HypKey
\arg \ref Ephy_G_VertexLabel

\HypIID
IID_hypIGeometricBase {2CC70D28-AB21-11d4-A098-0080AD97951B}
*/
struct IGeometricBase :
hyp_ker::IUnknown
{
	/*!
	Gets the number of the associated vertices.
	*/
	virtual t_size GetNumberOfVertices() =0;
	/*!
	Gets the vertex which has the specified label.
	\param Label Vertex Label of the vertex
	*/
	virtual IGeometricVertex* GetVertex(const t_label& Label) =0;
	/*!
	Gets the vertex which has the specified coordinates.
	\param X Coordinate of the vertex along the X-axe
	\param Y Coordiante of the vertex along the Y-axe
	\param Z Coodinate of the vertex along the Z-axe	
	*/
	virtual IGeometricVertex* GetVertex(const t_real& X,const t_real& Y,const t_real& Z) =0;
	/*!
	Gets the label of the vertex.
	\invariant The implementation generate an error if the vertex is not associated with the IGeometricBase object.
	*/
	virtual t_label GetLabel(IGeometricVertex* pVertex) =0;
	/*!
	Adds a vertex to the IGeometricBase object.
	\param pVertex Vertex which is added
	\param Label Vertex Label of the new inserted vertex
	*/
	virtual void AddVertex(IGeometricVertex* pVertex,const t_label& Label) =0;
	/*!
	Deletes the vertex associated to the IGeometricBase object
	*/
	virtual void DeleteVertex(const t_label& Label) =0;
	/*!
	Gives the IContainerKey objects which handles the vertices.
	\remark The implmentation is not strongly imposed to implement this service.
	*/
	virtual IContainerKey* GetContainerOfVertices() =0;
	/*!
	Adds a new filter. If the user want inserted a new vertex afterwards, a error will be
	generated only whether the vertex doesn't support all the interfaces of the filter.
	\remarks The implementation could memorize the last call of this service.
	\param pIID pointer to the Interface ID 
	*/
	virtual void SetFilter(const hyp_ker::hypIID* pIID) =0;
	/*!
	Clears the filter.
	*/
	virtual void ClearFilter() =0;
};

hyp_DEFINE_GUID(IID_hypIGeometricBase, 
0x2cc70d28, 0xab21, 0x11d4, 0xa0, 0x98, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IGeometricBase*/
typedef hyp_ker::IPtr<IGeometricBase,&IID_hypIGeometricBase> t_spGeometricBase;

/*!
\HypGeomData
This interface provides a set of services to describe a \e side.

\remarks In this version, this interface doesn't extend IGeometricBase. But has been
defined for an easier future extend.

\HypInvariant

\HypKey

\HypIID
IID_hypIGeometricSide {196913E6-18BD-11d5-A08F-0080AD97951B}
*/
struct IGeometricSide :
public hyp_ker::IUnknown
{
	
};

hyp_DEFINE_GUID(IID_hypIGeometricSide, 
0x196913e6, 0x18bd, 0x11d5, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IGeometricSide*/
typedef hyp_ker::IPtr<IGeometricSide,&IID_hypIGeometricSide> t_spGeometricSide;

/*!
\HypGeomData
Defines a set of services which handles :
\arg a collection of sides;
\arg the creation of new sides and new vertices.

Again, the sides in the collection is identified by a unique label called \ref Ephy_G_SideLabel.

\HypInvariant

\HypKey
\arg \ref Ephy_G_SideLabel

\HypIID
IID_hypIGeometricObject {196913E2-18BD-11d5-A08F-0080AD97951B}
*/
struct IGeometricObject :
public hyp_ker::IUnknown
{
	/*!
	Creates a new side. The created sides is attached to the IGeometricObject object.
	Moreover the side attaches a cluster of vertices. The vertices must be inserted in the IGeometricObject object.
	\param Label Side label of the created side
	\param LabelEnum Enumeration of Vertex Labels which must be attached to the created side.
	\todo The order of the enumeration is important 
	*/
	virtual IGeometricSide* CreateSide(const t_label& Label,const t_label_enum& LabelEnum) =0;
	/*!
	\overload
	This side doesn't have vertices.	
	*/
	virtual IGeometricSide* CreateSide(const t_label& Label) =0;
	/*!
	Creates a new vertex. The created vertex is attached to the IGeometricObject object.
	\param Label Vertex label of the created vertex
	\param X Coordianate of the vertex along the X-axe
	\param Y Coordinate of the vertex along the Y-axe
	\param Z Coodinate of the vertex along the Z-axe
	*/
	virtual IGeometricVertex* CreateVertex(const t_label& Label,const t_real& X,const t_real& Y,const t_real& Z) =0;
	/*!
	\overload
	The coordinates of the vertex are not explicitly initialized.
	\invariant The implmentation of the vertex object must initialize the coordinate at the creation. For instance {0,0,0}
	*/
	virtual IGeometricVertex* CreateVertex(const t_label& Label) =0;
	/*!
	Adds a side to the component.
	\param pObject Side added
	\param Label Label of the side inserted
	*/		
	virtual void AddSide(IGeometricSide*,const t_label& Label) =0;
	/*!
	Delates a side associated to a label.
	\param Label Label of the side
	*/
	virtual void DeleteSide(const t_label& Label) =0;
	/*!
	Gets the container of sides.
	*/
	virtual IContainerKey* GetContainerOfSides() =0;
	/*!
	Gets the numer of sides attached to the component.
	*/
	virtual t_size GetNumberOfSides() =0;
	/*!
	Gets the side associated to the label.
	\param Label Label of the side
	*/
	virtual IGeometricSide* GetSide(const t_label& Side) =0;	
};

hyp_DEFINE_GUID(IID_hypIGeometricObject, 
0x196913e2, 0x18bd, 0x11d5, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IGeometricBase*/
typedef hyp_ker::IPtr<IGeometricObject,&IID_hypIGeometricObject> t_spGeometricObject;
/*!@}*/
//-----------------------------------------------------------------------

//-----------------------------------------------------------------------
/*! \defgroup Ephy_Interface_FEO FEO Interfaces
\ingroup Ephy_Interface
@{
*/

/*!
The full description of a physical problem includes the geometrical description
(see \ref Ephy_G_GeometricData) as well as the description of the boundary conditions (BC).

For instance it is :
\arg the Degrees of Freedoms (DOF) of a element;
\arg the Forces which are applied on this such element.

The interface provides a set of services to access and to define the BC.

\HypInvariant

\HypKey

\HypIID
IID_hypIFEOBoundaryConditions {2CC70D24-AB21-11d4-A098-0080AD97951B}
*/
struct IFEOBoundaryConditions :
public hyp_ker::IUnknown
{
	/*!
	Fixs one of the DOF of the component. The DOF is identified by a Coordinate Label.
	For instance, if Coodinate Label is equal to 0,	the component is fixed along the X-axe.
	\param Label Coordinate label of the DOF
	*/
	virtual void FixDOF(const t_size& Label) =0;
	/*!
	Removes one of the DOF of the component.
	\param Label Coodinate label of the DOF
	*/
	virtual void UnFixDOF(const t_size& Label) =0;
	/*!
	Removes all the DOF associated to the component.
	*/
	virtual void ClearDOF() =0;
	/*!
	Checks if a DOF, characterized by its coordinate label, is associated to the component.
	\param Label Coordinate label
	*/
	virtual bool IsFixed(const t_size& Label) const =0;
};

hyp_DEFINE_GUID(IID_hypIFEOBoundaryConditions, 
0x2cc70d24, 0xab21, 0x11d4, 0xa0, 0x98, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IFEOBoundaryConditions */
typedef hyp_ker::IPtr<IFEOBoundaryConditions,&IID_hypIFEOBoundaryConditions> t_spFEOBoundaryConditions;

/*!
Each body or part of the body is characterized by a material:
\arg steel;
\arg concrete;
\arg air...

Each material could be described by a set of numerical data.
So this interface provides a subset of figures which are essential for the study of the deformation of solids.

\HypInvariant

\HypKey

\HypIID
IID_hypIFEOMaterial {196913E3-18BD-11d5-A08F-0080AD97951B}
*/
struct IFEOMaterial :
public hyp_ker::IUnknown
{
	/*!
	Sets the Young Modulus of the component.
	\param Scalar Young Modulus
	*/	
	virtual void SetYoung(const t_real& Scalar) =0;
	/*!
	Gets the Young Modulus of the component.	
	*/
	virtual t_real GetYoung() const =0;
	/*!
	Sets the Poisson Coefficient of the component.
	\param Scalar Poisson coefficient
	*/
	virtual void SetPoisson(const t_real& Scalar) =0;
	/*!
	Gets the POisson Coefficient of the component.
	*/
	virtual t_real GetPoisson() const =0;
};

hyp_DEFINE_GUID(IID_hypIFEOMaterial, 
0x196913e3, 0x18bd, 0x11d5, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IFEOMaterial*/
typedef hyp_ker::IPtr<IFEOMaterial,&IID_hypIFEOMaterial> t_spFEOMaterial;

/*!
Defines a finite element.

\HypInvariant

\HypKey

\HypIID
IID_hypIFEOElement {196913E1-18BD-11d5-A08F-0080AD97951B}
*/
struct IFEOElement :
public hyp_ker::IUnknown
{
	/*!
	Gets the material associated to the component.
	*/
	virtual const IFEOMaterial* GetMaterial() const =0;
	/*!
	Sets the material of the component.
	\param pObject Material
	*/
	virtual void SetMaterial(IFEOMaterial* pObject) =0;
	/*!
	Initializes the matrix representing the element.
	\invariant The size of the matrix must be checked by the implementation to fit to 
	the element matrix. The implmentation can return a error code or overwrite the size of the 
	matrix.
	*/
	virtual void SetKe(IMatrix* pObject) =0;
};

hyp_DEFINE_GUID(IID_hypIFEOElement, 
0x196913e1, 0x18bd, 0x11d5, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IFEOElement*/
typedef hyp_ker::IPtr<IFEOElement,&IID_hypIFEOElement> t_spFEOElement;

/*!
One of the main interfaces from the Ephy_G_Ephydryne.
The \e mesh is divided into several elements. Each elements are identified by an unique
label called \ref Ephy_G_ElementLabel.

\HypInvariant

\HypKey

\HypIID
IID_hypIFEOMesh {196913E4-18BD-11d5-A08F-0080AD97951B}
*/
struct IFEOMesh :
public hyp_ker::IUnknown
{
	/*!
	Creates an element and attach to the mesh. The created element attaches the vertices
	which are been given in parameters.
	\param Label Element label
	\param LabelEnum Enumeration of Vertex Label attached to the mesh
	\invariant The vertices must support the interface IFEOBoundaryConditions. Moreover the
	elements must be attached to the mesh.
	*/
	virtual IGeometricBase* CreateElement(const t_label& Label,const t_label_enum& LabelEnum) =0;
	/*!
	Gets the container of elements.
	*/
	virtual IContainerKey* GetContainerOfElements() =0; 
	/*!
	Gets the number of elements associated to the mesh.
	*/
	virtual t_size GetNumberOfElements() =0;
	/*!
	Gets the element associated to the mesh.
	\param Label Element label
	*/
	virtual IGeometricBase* GetElement(const t_label& Label) =0;
	/*!
	Removes all the components attached to the mesh. According to implementation, it is:
	- vertices
	- sides
	- elements
	- library of materials
	- etc.
	*/
	virtual void ClearAllData() =0;
	/*!
	Sets the default material associated to the mesh.
	\param pObject Default material
	\invariant This handy function can be coupled to the CreateElement function. The implementation
	can associated the default material to any new created element.	
	*/	
	virtual void SetDefaultMaterial(IFEOMaterial* pObject) =0;
	/*!
	Gets the default material.
	*/
	virtual const IFEOMaterial* GetDefaultMaterial() const =0;
	/*!
	Constructs the global matrix.
	*/
	virtual void ConstructGlobalMatrix() =0;
	/*!
	Gets the matrix associated to the mesh.
	*/
	virtual IMatrix* K() const =0;
	/*!
	Constructs the boundary matrix:
	*/
	virtual void ConstructBoundaryMatrix() =0;
	/*!
	Inverses the boundary matrix.
	*/
	virtual void InverseBoundaryMatrix() =0;
	/*!
	Constructs the visible matrix.
	*/
	virtual void ConstructVisibleMatrix() =0;
	/*!
	Inits the masks.
	*/
	virtual void InitMasks() =0;
	/*!
	Sets the render mask.
	\param Mask Render mask
	*/
	virtual void SetRenderMask(const t_mask_array& Mask) =0;
	/*!
	Gets the render mask.
	*/
	virtual t_mask_array RenderMask() =0;
	/*!
	Gets the displacement vector.
	*/
	virtual t_real_array Displacement() =0;
	/*!
	Gets the global displacement vector.
	*/
	virtual t_real_array GetGlobalDisplacement() =0;
	/*!
	Sets the real time mask.
	\param Mask Real-time mask
	*/	
	virtual void SetRealTimeMask(const t_mask_array& Mask) =0;
	/*!
	Gets the real time mask.
	*/
	virtual t_mask_array RealTimeMask() =0;
	/*!
	Gets the force vector.
	*/
	virtual t_real_array RealTimeForce() =0;
	/*!
	Gets the global force vector.
	*/
	virtual t_real_array GetGlobalRealTimeForce() =0;
	/*!
	Sets the global force vector.
	\param Array Global force vector
	*/
	virtual void SetGlobalRealTimeForce(const t_real_array& Array) =0;
	/*!
	Clears the force vector.
	*/
	virtual void ClearRealTimeForce() =0;
	/*!
	Starts the pre-render step.
	*/	
	virtual void PreRender() =0;
	/*!
	Excecutes the render step.
	*/
	virtual void Render() =0;
	/*!
	Excecutes the post-render step.
	*/
	virtual void PostRender() =0;
	/*!
	Sets a ponctual forces localized on a node.
	\param Label Node Label
	\param FX Value of the vector along the X-axe
	\param FY Value of the vector along the Y-axe
	\param FZ Value of the vector along the Z-axe
	*/
	virtual void SetForce(const t_label& Label,const t_real& FX,const t_real& FY,const t_real& FZ) =0;
	/*!
	Sets a ponctual force localized on the nodes matching the filter.
	\overload
	\param X If FilterX is true, the node which its coordinate along the X-axe is equal to X matches to the filter
	\param Y If FilterY is true, the node which its coordinate along the Y-axe is equal to Y matches to the filter
	\param Z If FilterZ is true, the node which its coordinate along the Z-axe is equal to Z matches to the filter
	\param FilterX Filter component
	\param FilterY Filter component
	\param FilterZ Filter component
	*/
	virtual void SetForce(const t_real& X,const t_real& Y,const t_real& Z,bool FilterX,bool FilterY,bool FilterZ,const t_real& FX,const t_real& FY,const t_real& FZ) =0;
	/*!
	Fixs the DOF of all the nodes matching the filter.
	\todo define a doxygen command to copy and paste
	\param DOFX if true, the matched node is fixed along the X-axe
	\param DOFY if true, the matched node is fixed along the Y-axe
	\param DOFZ if true, the matched node is fixed along the Z-axe
	*/
	virtual void FixDOF(const t_real& X,const t_real& Y,const t_real& Z,bool FilterX,bool FilterY,bool FilterZ,bool DOFX,bool DOFY,bool DOFZ) =0;
	/*!
	Checks if a side is fixed in a particualr direction.
	\param SideLabel Side label
	\param CoordinateLabel Coordinate label
	*/
	virtual bool IsSideFixed(const t_label& SideLabel,const t_size& CoordinateLabel) =0;
	
};

hyp_DEFINE_GUID(IID_hypIFEOMesh, 
0x196913e4, 0x18bd, 0x11d5, 0xa0, 0x8f, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSO IFEOMesh*/
typedef hyp_ker::IPtr<IFEOMesh,&IID_hypIFEOMesh> t_spFEOMesh;

/*!@}*/


hyp_NAMESPACE_END_HYPFEM

#endif