/*
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
 Ephydryne Components.
 Ephydryne is a part of the Hyperion Project.
----------------------------------------------------------
 Copyright (c) 2001 Pierre Rebours.

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 See hyp.license.txt file for more information about the license.
 
 Web: http://lagrandeporte.multimania.com/hyperion
 Email: pierre_rebours@yahoo.com
----------------------------------------------------------
 CVS
 $Log$
\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
*/


#ifndef _hyp_out_interfaces_h_
#define _hyp_out_interfaces_h_

#include<d3drmwin.h>

#define hyp_NAMESPACE_BEGIN_HYPOUT hyp_NAMESPACE_BEGIN_HYP \
										hyp_NAMESPACE_BEGIN(hypout)
#define hyp_NAMESPACE_END_HYPOUT hyp_NAMESPACE_END_HYP \
										hyp_NAMESPACE_END(hypout)


hyp_NAMESPACE_BEGIN_HYPOUT


//-----------------------------------------------------------------------
/*! \defgroup Ephy_Interface_Out Out Interfaces
\ingroup Ephy_Interface
@{
*/

/*!
Base interface. Each out components has got a Component-Document which must be
serialize or unserialize.
\invariant
The implmentation must checked if the cComponent-Documentn supports all the required interfaces.
If it is not the case, an error is thrown.
\HypIID IID_hypID3DMeshObject {CC11C7C1-9F7E-11d4-A08F-0080AD97951B}
*/
struct IFileObject :
public hyp_ker::IUnknown
{
	/*!
	Associates a Component-Document to the component.
	\param pObject Component-Document
	*/
	virtual void SetObject(hyp_ker::IUnknown* pObject) =0;
};


/*!
Provides a set of methods to deal with the DirectX API. Refers to the \ref Ephy_ExtLibrary_DirectX library
for more specific information.
\HypIID IID_hypID3DMeshObject {DD182BE1-375E-11d5-A099-0080AD97951B}
*/
struct ID3DMeshObject :
public IFileObject
{
	/*!
	Initialize the Component-Document with a LPDIRECT3DRMMESHBUILDER3 component.
	\param pComObject DirectX component which support the LPDIRECT3DRMMESHBUILDER3 interface.
	*/
	virtual void InitializeD3DMeshBuilder(LPDIRECT3DRMMESHBUILDER3 pComObject) =0;
	/*!
	Saves the Component-Document into a file. The saved data are only relates to the geometry
	of the Component-Document.
	\param FileName Name of the file
	*/
	virtual void SaveD3DMeshBuilder(const std::string& FileName) =0;
	/*!
	Saves the Component-Document into a file. The saved data include :
	- the geometry of the Component-Document
	- the matrix associated
	- the descriptions of the elements
	\param FileName Name of the file
	*/
	virtual void SaveD3DHypMesh(const std::string& FileName)=0;
	/*!
	Loads the Component-File from a file. This file has bee generated with the \ref SaveD3DHypMesh function.
	\param FileName Name of the file
	*/
	virtual void LoadD3DHypMesh(const std::string& FileName)=0;
};

// {DD182BE1-375E-11d5-A099-0080AD97951B}
hyp_DEFINE_GUID(IID_hypID3DMeshObject, 
0xdd182be1, 0x375e, 0x11d5, 0xa0, 0x99, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP ID3DMeshObject*/
typedef hyp_ker::IPtr<ID3DMeshObject,&IID_hypID3DMeshObject> t_spD3DMeshObject;




/*!
Provides a set of methods to write in a stream the content of the Component-Document,
\HypIID IID_hypIShowMeshObject {8DD90FE1-206B-11d5-A096-0080AD97951B}
*/
struct IShowMeshObject :
public IFileObject
{
	/*!
	Associates a stream to the component. The implmentation will open a stream from the file.
	\param FileName File associated to the component
	*/
	virtual void SetOutput(const std::string& FileName) =0;
	/*!
	\overload
	\param Stream	Stream associated to the component
	*/
	virtual void SetOutput(const std::ostream& Stream) =0;
	/*!
	Closes the stream associated to the component.
	*/
	virtual void CloseOutput() =0;
	/*!
	Writes the description of the nodes in the stream.
	*/	
	virtual void ShowNodes() =0;
	/*!
	Writes the description of the sides in the stream.
	*/
	virtual void ShowSides() =0;
	/*!
	Writes the description of the sides in the stream.
	*/
	virtual void ShowElements() =0;
	/*!
	Writes the description of the matrix in the stream.
	*/
	virtual void ShowMatrix() =0;
	/*!
	Writes the dimension of the matrix in the stream. It sometimes uses to replace the
	huge overhead of the writing of the matrix.
	*/
	virtual void ShowMatrixDim() =0;
	/*!
	Writes the description of the Force Mask in the stream.
	*/
	virtual void ShowForceMask() =0;
	/*!
	Writes the description of the Render Mask in the stream.
	*/
	virtual void ShowRenderMask() =0;
	/*!
	Writes the displacement of the nodes in the stream.
	*/
	virtual void ShowDisplacements() =0;
	/*!
	Writes the global displacement of the nodes in the stream.
	*/
	virtual void ShowGlobalDisplacements() =0;
	/*!
	Writes the forces applied to the nodes in the stream.
	*/
	virtual void ShowForces() =0;
	/*!
	Writes the global forces applied to the nodes in the stream.
	*/
	virtual void ShowGlobalForces() =0;
	/*!
	Writes all the revelant information concerning the Component-Document in the stream.
	*/
	virtual void ShowAll() =0;
};

// {8DD90FE1-206B-11d5-A096-0080AD97951B}
hyp_DEFINE_GUID(IID_hypIShowMeshObject, 
0x8dd90fe1, 0x206b, 0x11d5, 0xa0, 0x96, 0x0, 0x80, 0xad, 0x97, 0x95, 0x1b);

/*! \HypSP IShowMeshObject*/
typedef hyp_ker::IPtr<IShowMeshObject,&IID_hypIShowMeshObject> t_spShowMeshObject;

/*!@}*/
//-------------------------------------------------------------------------

hyp_NAMESPACE_END_HYPOUT

#endif